/*
  Java Japanese-English dictionary/Kanjidic browser
  Copyright (C) 1997 Todd David Rudick

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

// Bushu-selection method GUI

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import GenList;
import KanjiButtonList;
import ButtonList;
import KanjiViewer;
import JavaDict;

class BushSelect extends Panel implements ButtonList.ButtonListListener {
  BushuGUI paps;
  static final String RadicalFile = "radicals.tm";
  public static Vector bushu; /* a vector of Strings
		   each String is: <Bushu><space><number>
		   index+1 into the vector represents stroke count */
  String lastAction = "";
  static char[] bushuChar = new char[214];
  
  public KanjiButtonList kanjilist;
  {
    init();
  }
  
  static void init()
  { // static initializer to load in the radicals.tm file
    /* first, init the Vector */
    try {
      bushu= new Vector();
      int strokes = 0;
      int i;
      for (i=0;i<17;i++) bushu.addElement(new Vector());

      File f = new File(RadicalFile);
      FileInputStream tempin = new FileInputStream(f);
      InputStreamReader tempin2 = new InputStreamReader(tempin,"EUCJIS");
      BufferedReader in = new BufferedReader(tempin2);
      String s;
      while ((s=in.readLine())!=null) {
	s = s.trim();
	int index;
	index = s.indexOf(" ");
	if (index!=-1) {
	  int rad;
	  rad=-1;
	  String stmp;
	  stmp = s.substring(index+1);
	  try {
	    rad = Integer.parseInt(stmp);
	  } catch (NumberFormatException nfe) {
	    String stmp2=null;
	    stmp2 = stmp.substring(1,stmp.length()-1);
	    rad = Integer.parseInt(stmp2);
	  }
	  if (rad==0) strokes++;
	  else {
	    ((Vector)bushu.elementAt(strokes-1)).addElement(s.charAt(0)+
							    " "+rad);
	    try {
	      bushuChar[rad-1] = s.charAt(0);
	    } catch (Exception arrexp) {
	      System.out.println("Unexpected radical #"+rad);
	    }
	  }
	}
      }
    } catch (IOException e) {
      System.out.println("Could not load file: " +RadicalFile+ ". exiting.");
      System.exit(1);
    }
  }

  public static char getBushuChar(int bushu) {
    try {
      return bushuChar[bushu-1];
    } catch (Exception e) {
      return '?';
    }
  }

  public void ActionText(String s) {
    //    System.out.println("Look up kanji with Bushu " + s);
    if (s.length()==0) return;
    lastAction = s;

    int index;
    index = s.indexOf(" ");
    int bushu;
    try {
      bushu = Integer.parseInt(s.substring(index+1));
    } catch (NumberFormatException nfe) {
      System.out.println(nfe.getMessage());
      return;
    }
    String bushuCode = "B"+bushu+" ";
    kanjilist.setallowrare(paps.allowRare.getState());
    java.lang.Thread t;
    t = new java.lang.Thread(JavaDict.LookUp.lookupgroup,
			     new JavaDict.LookUp(bushuCode,
						 kanjilist,
						 paps.dict, 
						 paps.dictindex,
						 false));
    t.setDaemon(true); // exit if it's the only thing running
    t.setPriority(java.lang.Thread.MIN_PRIORITY);
    
    t.start();
  }

  public BushSelect(BushuGUI paps) {
    this.paps=paps;
    /* Bushu selection screen looks like:
          | Bushu    |  Kanji    |
          |----------------------|
          | 1 stroke |     &     |
          |    x     | 4 strokes |
          |    -     |     /     |
          |    /     |     @     |
          |    ^     |     %     |         */
    setLayout(new GridLayout(1,2));
    Panel LeftPanel = new Panel(new BorderLayout());

    LeftPanel.add(new Label("Bushu",Label.CENTER), "North");
    Panel RightPanel = new Panel(new BorderLayout());

    RightPanel.add(new Label("Kanji",Label.CENTER), "North");
    
    ButtonList bushuList = new ButtonList(2,this);
    bushuList.setFont(paps.BigFont);
    LeftPanel.add(bushuList,"Center");

    kanjilist = new KanjiButtonList(2,paps.javadict.kanjiview,
				    paps.allowRare.getState());
    kanjilist.showStrokes = true;
    kanjilist.showColors = false;
    kanjilist.setFont(paps.BigFont);
    RightPanel.add(kanjilist,"Center");

    Enumeration e = bushu.elements();
    int strokes = 0;
    while (e.hasMoreElements()) {
      Vector v = (Vector)e.nextElement();
      strokes++;
      /*      if (strokes>1) bushuList.addLabel(strokes+" strokes"); else
	bushuList.add(strokes+" stroke");*/
      bushuList.addLabel(strokes+"",Color.white, Color.black);
      Enumeration e2 = v.elements();
      while (e2.hasMoreElements()) {
	String s;
	s = (String) e2.nextElement();
	int index;
	index = s.indexOf(" ");
	if (index!=-1) {
	  //bushuList.add(s.substring(0,index));
	  bushuList.add(s);
	}
      }
    }
    add(LeftPanel);
    add(RightPanel);
  }
  public void reSearch() {
    ActionText(lastAction);
  }
}

public class BushuGUI extends Panel implements ItemListener {
  public Font BigFont;
  public JavaDict javadict;
  public String dict, dictindex;
  public boolean allowRareB;
  Checkbox allowRare;
  BushSelect bushSelect;

  public static class Bushu2Char implements KanjiButtonList.BushuProvider {
    
    public Bushu2Char() {
    }
    public char getBushuChar(int bushu) {
      return(BushSelect.getBushuChar(bushu)); 
	     // this gets around static method in 
	     // interface error crapola (i.e., they're not allowed)
    }
  }

  public void itemStateChanged(ItemEvent e) { // the checkbox has been hit
    allowRareB = allowRare.getState();
    // repeat the search
    bushSelect.reSearch();
  }

  public BushuGUI(JavaDict javadict, Button MenuButton,
		  String dict, String dictindex) {
    this.BigFont = javadict.BigFont;
    this.javadict=javadict;
    this.dict=dict;
    this.dictindex=dictindex;
    
    setLayout(new BorderLayout());

    Panel allowRareP = new Panel();

    allowRare = new Checkbox("Allow Rare Kanji");
    allowRare.addItemListener(this);
    allowRareB = allowRare.getState();

    Panel inputPanel = new Panel(new BorderLayout());
    add(inputPanel,"South");
    Panel buttonPanel = new Panel(new BorderLayout());
    inputPanel.add(buttonPanel,"East");
    buttonPanel.add(MenuButton,"South");

    /*    Panel cboxPanel = new Panel(new BorderLayout());
    
	  inputPanel.add(cboxPanel,"Center");
	  cboxPanel.add(allowRare,"East");*/
    inputPanel.add(allowRare,"West");
    
    bushSelect = new BushSelect(this);
    add(bushSelect,"Center");
  }
}
