/*
  Java Japanese-English dictionary/Kanjidic browser
  Copyright (C) 1997 Todd David Rudick

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

// B-M search class
public class ByteSearch {
  int[] delta;
  byte[] pattern;
  int patlen;

  // pass this a 0 terminated or correct length array of bytes
  public ByteSearch(byte[] pattern) {
    delta = new int[257];
    this.pattern= new byte[pattern.length];
    for (patlen=0;(patlen<pattern.length)&&(pattern[patlen]!=0);patlen++) {
      this.pattern[patlen]=pattern[patlen];
    }
    // compute deltas
    int i;
    for (i=0;i<257;i++) delta[i]=patlen;
    for (i=0;i<patlen;i++)
      delta[pattern[i]+128]=patlen-1-i;
  }

  // do search in buffer, return index of match or -1
  // "end" is inclusive, but matches will lie ONLY start<=match<=end
  // Routine requires (patlen) extra bytes after "end" (which are restored)
  public int search(byte[] buffer, int start,int end) {
    //    System.out.println("Starting search routine");
    int index;
    index=start+patlen-1;
    // place pattern at end
    int i;
    byte[] tempbuf = new byte[patlen];
    for (i=0;i<patlen;i++) {
      tempbuf[i]=buffer[end+1+i];
      buffer[end+1+i]=pattern[i];
    }
    while (true) {
      index+=delta[buffer[index]+128];
      index+=delta[buffer[index]+128];
      index+=delta[buffer[index]+128];
      if (delta[buffer[index]+128]==0) {
	// check for a hit
	boolean goodhit=true;
	int cstart;
	cstart = index-(patlen-1);
	for (i=cstart;i<index;i++) 
	  goodhit=(goodhit)&&(pattern[i-cstart]==buffer[i]);
	if (goodhit) {
	  for (i=0;i<patlen;i++) 
	    buffer[end+1+i]=tempbuf[i];
	  if (index>end) {
	    //	    System.out.println("ending search routine");
	    return(-1);
	  }
	  else {
	    //	    System.out.println("ending search routine");
	    return(index-(patlen-1));
	  }
	} else index++;
      }
    }
  }

  static public byte[] getLine(byte[] buffer,
			       int position) {
    int retlen=0;
    int preindex;
    preindex = position-1;
    while (buffer[preindex]!=012) preindex--;
    int postindex;
    postindex = position+1;
    while (buffer[postindex]!=012) postindex++;
    int retsize;
    retsize = postindex-preindex-1;
    byte[] retbuf = new byte[retsize];
    int i;
    for (i=0;i<retsize;i++) retbuf[i]=buffer[preindex+1+i];
    return(retbuf);
  }
}
