/*
  Java Japanese-English dictionary/Kanjidic browser
  Copyright (C) 1997 Todd David Rudick

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import GenList;
import KanjiButtonList;
import KanjiViewer;

class ImageCanvas extends Canvas implements MouseListener, KeyListener {
  Container pappy;
  Image image;
  boolean trueSizeKnown = false;
  Dimension minSize;
  int w, h;
  int iw,ih;
  boolean haveFocus = false;
  ActionListener aListener;
  String ACCOMMAND = "click!";
  
  public ImageCanvas(Image image, Container parent, 
		     int initialWidth, int initialHeight,
		     ActionListener a) {
    aListener = a;
    if (image == null) {
      System.err.println("Canvas got invalid image object!");
      return;
    }
    
    this.image = image;
    pappy = parent;
    
    w = iw = initialWidth;
    h = ih = initialHeight;
    
    minSize = new Dimension(w,h);
    enableEvents(AWTEvent.FOCUS_EVENT_MASK);
    if (aListener!=null) {
      addMouseListener(this);
      addKeyListener(this);
    }
  }

  public boolean isTabbable() {
    return (aListener!=null);
  }
  
  public boolean isFocusTraversable() {
    return (aListener!=null);
  }

  public void keyTyped(KeyEvent e) { }
  public void keyReleased(KeyEvent e) { }
  public void keyPressed(KeyEvent e) {
    if (e.getKeyCode()==KeyEvent.VK_SPACE) {
      mousePressed(null);
    }
  }

  public void mouseEntered(MouseEvent e) { }
  public void mouseExited(MouseEvent e) { }
  public void mouseReleased(MouseEvent e) { }
  public void mouseClicked(MouseEvent e) { }

  public void mousePressed(MouseEvent e) { 
    ActionEvent a = new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
				    ACCOMMAND);
    aListener.actionPerformed(a);
  }

  protected void processFocusEvent(FocusEvent e) {
    switch(e.getID()) {
    case FocusEvent.FOCUS_GAINED:
      haveFocus = true;
      break;
    case FocusEvent.FOCUS_LOST:
      haveFocus = false;
    }
    repaint();
    super.processFocusEvent(e);
  }

  public void setImage(Image image) {
    if (image == null) {
      System.err.println("Canvas got invalid image object!");
      return;
    }
    this.image=image;
    if (!((w==image.getWidth(this))&&(h==image.getHeight(this)))) {
      w = iw;
      h = ih;
    
      minSize = new Dimension(w,h);
      trueSizeKnown=false;
    }
    Graphics g = getGraphics();
    if (g!=null)
      paint(g);
  }
  
/*  public Dimension preferredSize() {
         return minimumSize();
  }    */
         /*
  public synchronized Dimension minimumSize() {
    return minSize;
  }        */
  
  public void paint (Graphics g) {
    if (image != null) {
      if (!trueSizeKnown) {
	int imageWidth = image.getWidth(this);
	int imageHeight = image.getHeight(this);
	
	if ((imageWidth > 0) && (imageHeight > 0)) {
	  trueSizeKnown = true;
	  
	  //Ooh... component-initiated resizing.
	  w = imageWidth;
	  h = imageHeight;
	  minSize = new Dimension(w,h);
	  setSize(w, h);
	  pappy.doLayout();
	  pappy.repaint();
	}
      }
      Rectangle r = getBounds();
      int xoff = (r.width - w)/2;
      int yoff = (r.height - h)/2;
      //      g.fill3DRect(r.x+3,r.y+3,r.width-6,r.height-6,true);
      g.setColor(getBackground());
      g.draw3DRect(xoff-1,yoff-1,w+2,h+2,true);
      g.drawImage(image, xoff, yoff, this);
      if (haveFocus) {
	g.setColor(Color.black);
	g.drawRect(xoff-1,yoff-1,w+2,h+2);
	g.drawRect(xoff,yoff,w,h);
      }
    }
  }
}



public class SkipGUI extends Panel implements ItemListener {
  public JavaDict javadict;
  public String dict;
  public String dictindex;
  //  public GenList.List OutputArea;
  public KanjiButtonList OutputArea;
  //  KanjiViewer kanjiview;
  Checkbox allowrare;
  FlipPanel flippanel;

  public void itemStateChanged(ItemEvent e) { // the checkbox has been hit
    flippanel.fixup();
  }

  public void paint(Graphics g) {
    super.paint(g);
    //    kanjiview.paint(g);
    OutputArea.paint(OutputArea.getGraphics());
  }

  public SkipGUI(JavaDict javadict, Button MenuButton,
		 String dict,
		 String dictindex) {

    Font BigFont = javadict.BigFont;

    this.javadict=javadict;
    this.dict=dict;
    this.dictindex=dictindex;
    //    CardLayout cardlayout = new CardLayout();
    //    this.setLayout(cardlayout);

    //    kanjiview = new KanjiViewer(this,cardlayout,"kanjiview",
    //				"MainPanel",
    //				javadict);
    //    this.add(kanjiview,"kanjiview");
    //    Panel f = new Panel();
    Panel f = this;
    //    this.add(f,"MainPanel");
    //    cardlayout.show(this,"MainPanel");
    f.setLayout(new BorderLayout());

    Panel allowrareP = new Panel();
    allowrare = new Checkbox("Allow Rare Kanji");
    allowrare.addItemListener(this);
    OutputArea = new KanjiButtonList(6,javadict.kanjiview,
				     allowrare.getState());
    OutputArea.showColors=false;
    OutputArea.bushuProvider = new BushuGUI.Bushu2Char();

      //GenList.List(6,false);
    OutputArea.setFont(BigFont);
    f.add(OutputArea,"Center");
    //    int i;
    //    for (i=0;i<30;i++)
    // OutputArea.add("Item "+i);
    
    Panel InputPanel = new Panel();
    InputPanel.setLayout(new BorderLayout());
    f.add(InputPanel,"South");

    flippanel = new FlipPanel(this);
    InputPanel.add(flippanel,"West");
    InputPanel.add(allowrareP,"Center");
    allowrareP.add(allowrare);
    Panel ButtonPanel = new Panel(new BorderLayout());
    InputPanel.add(ButtonPanel,"East");
    ButtonPanel.add(MenuButton,"South");
  }

  static class FlipPanel extends Panel {
    public SkipGUI paps;
    public Image[] T4 = new Image[4];
    public Image[] Shape = new Image[4];
    public Image UpArrow;
    public Image DownArrow;
    public int[] index = new int[3];
    public ImageCanvas b1,b2,b3,b4,b5,b6; /* three up, three down */
    public Label l7,l8,l9; /* descriptive labels */
    public ImageCanvas shape; /* = skip #1 */
    public Label Val2; /* = skip #2 */
    public CardLayout Skip3Card; /* = skip #3 card */
    public Panel Skip3Panel;
    public ImageCanvas Skip3Shape;
    public Label Val3;
    boolean firsttime = true;

    public int[] SkipVal = {1,1,1};

    public void paint(Graphics g) {
      super.paint(g);
      if (firsttime) {
	firsttime=false;
	fixup();
      }
    }

    // Changes all the labels and stuff depending on SkipVal[]
    public void fixup() {
      shape.setImage(Shape[SkipVal[0]-1]);
      Val2.setText(""+SkipVal[1]);
      if (SkipVal[0]!=4) {
	Skip3Card.show(Skip3Panel,"Number");
	Val3.setText(""+SkipVal[2]);
      } 
      else {
	Skip3Card.show(Skip3Panel,"Shape");
	Skip3Shape.setImage(T4[SkipVal[2]-1]);
      }
      // handle labels
      switch (SkipVal[0]) {
      case 1:
	l8.setText("Left");
	l9.setText("Right");
	break;
      case 2:
	l8.setText("Top");
	l9.setText("Bottom");
	break;
      case 3:
	l8.setText("Outside");
	l9.setText("Inside");
	break;
      case 4:
	l8.setText("Strokes");
	l9.setText("Line");
	break;
      }
      Graphics g;
      g = getGraphics();
      if (g!=null) {
	paintAll(g);
	String skipCode;
	skipCode = "P" + SkipVal[0] + "-" + SkipVal[1] + "-" + 
	  SkipVal[2]+" ";
	// check the state of the "rare kanji" checkbox and let the buttonlist
	// know whether to let them be added or not
	// (this is sort of a race condition with a possible current search,
	//  but it'll be killed and the list cleared ASAP anyway, 
	// so it does no harm)
	paps.OutputArea.setallowrare(paps.allowrare.getState());
	java.lang.Thread t;
	t = new java.lang.Thread(JavaDict.LookUp.lookupgroup,
				 new JavaDict.LookUp(skipCode,
						     paps.OutputArea,
						     paps.dict, 
						     paps.dictindex,false));
	t.setDaemon(true); // exit if it's the only thing running
	t.setPriority(java.lang.Thread.MIN_PRIORITY);
	
	t.start();
      }

    }

    static class FlipPanelListen implements ActionListener {
      int ButNum;
      FlipPanel paps;

      public FlipPanelListen(int ButNum, FlipPanel paps) {
	this.ButNum = ButNum;
	this.paps = paps;
      }
      public void actionPerformed (ActionEvent e) { 
	switch (ButNum) { 
	  // ups
	case 1:
	  if (paps.SkipVal[0]==4) paps.SkipVal[2]=1;
	  paps.SkipVal[0]=(paps.SkipVal[0]%4)+1;
	  if (paps.SkipVal[0]==4) paps.SkipVal[2]=1;
	  break;
	case 2:
	  paps.SkipVal[1]++;
	  break;
	case 3:
	  if (paps.SkipVal[0]!=4) {
	    paps.SkipVal[2]++;
	  } else {
	    if (paps.SkipVal[2]<4) paps.SkipVal[2]++;
	  }
	  break;
	  // downs
	case 4:
	  if (paps.SkipVal[0]==4) paps.SkipVal[2]=1;
	  paps.SkipVal[0]=((paps.SkipVal[0]+2)%4)+1;
	  if (paps.SkipVal[0]==4) paps.SkipVal[2]=1;
	  break;
	case 5:
	  if (paps.SkipVal[1]>1)
	    paps.SkipVal[1]--;
	  break;
	case 6:
	  if (paps.SkipVal[2]>1)
	    paps.SkipVal[2]--;
	  break;
	}
	paps.fixup();
      }
    }

    public FlipPanel(SkipGUI paps) {
      this.paps = paps;
      Shape[0]=getToolkit().getImage("SkipShape1.gif");
      Shape[1]=getToolkit().getImage("SkipShape2.gif");
      Shape[2]=getToolkit().getImage("SkipShape3.gif");
      Shape[3]=getToolkit().getImage("SkipShape4.gif");
      T4[0]=getToolkit().getImage("solid1.gif");
      T4[1]=getToolkit().getImage("solid2.gif");
      T4[2]=getToolkit().getImage("solid3.gif");
      T4[3]=getToolkit().getImage("solid4.gif");
      UpArrow=getToolkit().getImage("uparrow.gif");
      DownArrow=getToolkit().getImage("downarrow.gif");
      
      setLayout(new GridLayout(3,6,0,0));

      // dummy top cols
      Label l1 = new Label("");
      Label l2 = new Label("");
      Label l3 = new Label("");

      // up buttons
      add(l1);
      b1 = new ImageCanvas(UpArrow,this,15,15,new FlipPanelListen(1,this));
      add(b1);
    
      add(l2);
      b2 = new ImageCanvas(UpArrow,this,15,15,new FlipPanelListen(2,this));
      add(b2);

      add(l3);
      b3 = new ImageCanvas(UpArrow,this,15,15,new FlipPanelListen(3,this));
      add(b3);

      // Middle buttons/entries
      l7 = new Label("Shape",Label.RIGHT);
      add(l7);
      shape = new ImageCanvas(Shape[0],this,24,24,null);
      add(shape);

      l8 = new Label("Outside",Label.RIGHT);
      add(l8);
      Val2 = new Label("1",Label.CENTER);
      add(Val2);

      l9 = new Label("Inside",Label.RIGHT);
      add(l9);

      Skip3Panel = new Panel();
      Skip3Card = new CardLayout();
      Skip3Panel.setLayout(Skip3Card);
      add(Skip3Panel);
      
      Skip3Shape = new ImageCanvas(T4[0],this,16,16,null);
      Skip3Panel.add(Skip3Shape,"Shape");

      Val3 = new Label("1",Label.CENTER);
      Skip3Panel.add(Val3,"Number");

      // dummy bottom cols.
      Label l4 = new Label("");
      Label l5 = new Label("");
      Label l6 = new Label("");

      // down buttons
      add(l4);
      b4 = new ImageCanvas(DownArrow,this,15,15,new FlipPanelListen(4,this));
      add(b4);

      add(l5);
      b5 = new ImageCanvas(DownArrow,this,15,15,new FlipPanelListen(5,this));
      add(b5);

      add(l6);
      b6 = new ImageCanvas(DownArrow,this,15,15,new FlipPanelListen(6,this));
      add(b6);
      fixup();
    }
  }
}

